@@ -11,22 +11,22 @@ import operator
 import types
 
 import six
-if six.PY3:
-    from .compat import is_bound_method as ismethod
-else:
-    from inspect import ismethod
 
 from webob import (Request as WebObRequest, Response as WebObResponse, exc,
                    acceptparse)
 from webob.multidict import NestedMultiDict
 
-from .compat import urlparse, unquote_plus, izip
+from .compat import urlparse, izip
 from .secure import handle_security
 from .templating import RendererFactory
 from .routing import lookup_controller, NonCanonicalPath
 from .util import _cfg, encode_if_needed, getargspec
 from .middleware.recursive import ForwardRequestException
 
+if six.PY3:
+    from .compat import is_bound_method as ismethod
+else:
+    from inspect import ismethod
 
 # make sure that json is defined in mimetypes
 add_type('application/json', '.json', True)
@@ -343,14 +343,12 @@ class PecanBase(object):
         args = []
         varargs = []
         kwargs = dict()
-        valid_args = argspec.args[1:]  # pop off `self`
+        valid_args = argspec.args[:]
+        if ismethod(state.controller) or im_self:
+            valid_args.pop(0)  # pop off `self`
         pecan_state = state.request.pecan
 
-        def _decode(x):
-            return unquote_plus(x) if isinstance(x, six.string_types) \
-                else x
-
-        remainder = [_decode(x) for x in remainder if x]
+        remainder = [x for x in remainder if x]
 
         if im_self is not None:
             args.append(im_self)
@@ -421,7 +419,7 @@ class PecanBase(object):
 
         # store the routing path for the current application to allow hooks to
         # modify it
-        pecan_state['routing_path'] = path = req.encget('PATH_INFO')
+        pecan_state['routing_path'] = path = req.path_info
 
         # handle "on_route" hooks
         self.handle_hooks(self.hooks, 'on_route', state)
@@ -536,7 +534,7 @@ class PecanBase(object):
         # handle "before" hooks
         self.handle_hooks(self.determine_hooks(controller), 'before', state)
 
-        return controller, args+varargs, kwargs
+        return controller, args + varargs, kwargs
 
     def invoke_controller(self, controller, args, kwargs, state):
         '''
