@@ -1,14 +1,13 @@
+# -*- coding: utf-8 -*-
+
 import sys
 import os
 import json
 import warnings
-if sys.version_info < (2, 7):
-    import unittest2 as unittest  # pragma: nocover
-else:
-    import unittest  # pragma: nocover
 
 import webob
 from webob.exc import HTTPNotFound
+import mock
 from webtest import TestApp
 import six
 from six import b as b_
@@ -17,7 +16,7 @@ from six.moves import cStringIO as StringIO
 
 from pecan import (
     Pecan, Request, Response, expose, request, response, redirect,
-    abort, make_app, override_template, render
+    abort, make_app, override_template, render, route
 )
 from pecan.templating import (
     _builtin_renderers as builtin_renderers, error_formatters
@@ -25,6 +24,11 @@ from pecan.templating import (
 from pecan.decorators import accept_noncanonical
 from pecan.tests import PecanTestCase
 
+if sys.version_info < (2, 7):
+    import unittest2 as unittest  # pragma: nocover
+else:
+    import unittest  # pragma: nocover
+
 
 class SampleRootController(object):
     pass
@@ -257,6 +261,35 @@ class TestObjectDispatch(PecanTestCase):
         assert r.body == b_('/sub/sub/deeper')
 
 
+@unittest.skipIf(not six.PY3, "tests are Python3 specific")
+class TestUnicodePathSegments(PecanTestCase):
+
+    def test_unicode_methods(self):
+        class RootController(object):
+            pass
+        setattr(RootController, '🌰', expose()(lambda self: 'Hello, World!'))
+        app = TestApp(Pecan(RootController()))
+
+        resp = app.get('/%F0%9F%8C%B0/')
+        assert resp.status_int == 200
+        assert resp.body == b_('Hello, World!')
+
+    def test_unicode_child(self):
+        class ChildController(object):
+            @expose()
+            def index(self):
+                return 'Hello, World!'
+
+        class RootController(object):
+            pass
+        setattr(RootController, '🌰', ChildController())
+        app = TestApp(Pecan(RootController()))
+
+        resp = app.get('/%F0%9F%8C%B0/')
+        assert resp.status_int == 200
+        assert resp.body == b_('Hello, World!')
+
+
 class TestLookups(PecanTestCase):
 
     @property
@@ -394,6 +427,11 @@ class TestControllerArguments(PecanTestCase):
                     ', '.join(list(args) + data)
                 )
 
+            @staticmethod
+            @expose()
+            def static(id):
+                return "id is %s" % id
+
             @expose()
             def _route(self, args, request):
                 if hasattr(self, args[0]):
@@ -424,6 +462,16 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('index: This is a test!')
 
+    def test_single_argument_with_plus(self):
+        r = self.app_.get('/foo+bar')
+        assert r.status_int == 200
+        assert r.body == b_('index: foo+bar')
+
+    def test_single_argument_with_encoded_plus(self):
+        r = self.app_.get('/foo%2Bbar')
+        assert r.status_int == 200
+        assert r.body == b_('index: foo+bar')
+
     def test_two_arguments(self):
         r = self.app_.get('/1/dummy', status=404)
         assert r.status_int == 404
@@ -438,6 +486,16 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('index: This is a test!')
 
+    def test_keyword_argument_with_plus(self):
+        r = self.app_.get('/?id=foo+bar')
+        assert r.status_int == 200
+        assert r.body == b_('index: foo bar')
+
+    def test_keyword_argument_with_encoded_plus(self):
+        r = self.app_.get('/?id=foo%2Bbar')
+        assert r.status_int == 200
+        assert r.body == b_('index: foo+bar')
+
     def test_argument_and_keyword_argument(self):
         r = self.app_.get('/3?id=three')
         assert r.status_int == 200
@@ -917,6 +975,11 @@ class TestControllerArguments(PecanTestCase):
         assert r.status_int == 200
         assert r.body == b_('variable_kwargs: list=%s' % l)
 
+    def test_staticmethod(self):
+        r = self.app_.get('/static/foobar')
+        assert r.status_int == 200
+        assert r.body == b_('id is foobar')
+
     def test_no_remainder(self):
         try:
             r = self.app_.get('/eater')
@@ -1605,6 +1668,41 @@ class TestNonCanonical(PecanTestCase):
         assert len(wrapped_apps) == 1
 
 
+class TestDebugging(PecanTestCase):
+    def test_debugger_setup(self):
+        class RootController(object):
+            pass
+
+        def debugger():
+            pass
+
+        app_conf = dict(
+            debug=True,
+            debugger=debugger
+        )
+        with mock.patch('pecan.middleware.debug.DebugMiddleware') \
+                as patched_debug_middleware:
+            app = make_app(RootController(), **app_conf)
+            args, kwargs = patched_debug_middleware.call_args
+            assert kwargs.get('debugger') == debugger
+
+    def test_invalid_debugger_setup(self):
+        class RootController(object):
+            pass
+
+        debugger = 'not_a_valid_entry_point'
+
+        app_conf = dict(
+            debug=True,
+            debugger=debugger
+        )
+        with mock.patch('pecan.middleware.debug.DebugMiddleware') \
+                as patched_debug_middleware:
+            app = make_app(RootController(), **app_conf)
+            args, kwargs = patched_debug_middleware.call_args
+            assert kwargs.get('debugger') is None
+
+
 class TestLogging(PecanTestCase):
 
     def test_logging_setup(self):
@@ -1867,3 +1965,264 @@ class TestDeprecatedRouteMethod(PecanTestCase):
             r = self.app_.get('/foo/bar/')
             assert r.status_int == 200
             assert b_('foo, bar') in r.body
+
+
+class TestExplicitRoute(PecanTestCase):
+
+    def test_alternate_route(self):
+
+        class RootController(object):
+
+            @expose(route='some-path')
+            def some_path(self):
+                return 'Hello, World!'
+
+        app = TestApp(Pecan(RootController()))
+
+        r = app.get('/some-path/')
+        assert r.status_int == 200
+        assert r.body == b_('Hello, World!')
+
+        r = app.get('/some_path/', expect_errors=True)
+        assert r.status_int == 404
+
+    def test_manual_route(self):
+
+        class SubController(object):
+
+            @expose(route='some-path')
+            def some_path(self):
+                return 'Hello, World!'
+
+        class RootController(object):
+            pass
+
+        route(RootController, 'some-controller', SubController())
+
+        app = TestApp(Pecan(RootController()))
+
+        r = app.get('/some-controller/some-path/')
+        assert r.status_int == 200
+        assert r.body == b_('Hello, World!')
+
+        r = app.get('/some-controller/some_path/', expect_errors=True)
+        assert r.status_int == 404
+
+    def test_manual_route_conflict(self):
+
+        class SubController(object):
+            pass
+
+        class RootController(object):
+
+            @expose()
+            def hello(self):
+                return 'Hello, World!'
+
+        self.assertRaises(
+            RuntimeError,
+            route,
+            RootController,
+            'hello',
+            SubController()
+        )
+
+    def test_custom_route_on_index(self):
+
+        class RootController(object):
+
+            @expose(route='some-path')
+            def index(self):
+                return 'Hello, World!'
+
+        app = TestApp(Pecan(RootController()))
+
+        r = app.get('/some-path/')
+        assert r.status_int == 200
+        assert r.body == b_('Hello, World!')
+
+        r = app.get('/')
+        assert r.status_int == 200
+        assert r.body == b_('Hello, World!')
+
+        r = app.get('/index/', expect_errors=True)
+        assert r.status_int == 404
+
+    def test_custom_route_with_attribute_conflict(self):
+
+        class RootController(object):
+
+            @expose(route='mock')
+            def greet(self):
+                return 'Hello, World!'
+
+            @expose()
+            def mock(self):
+                return 'You are not worthy!'
+
+        app = TestApp(Pecan(RootController()))
+
+        self.assertRaises(
+            RuntimeError,
+            app.get,
+            '/mock/'
+        )
+
+    def test_conflicting_custom_routes(self):
+
+        class RootController(object):
+
+            @expose(route='testing')
+            def foo(self):
+                return 'Foo!'
+
+            @expose(route='testing')
+            def bar(self):
+                return 'Bar!'
+
+        app = TestApp(Pecan(RootController()))
+
+        self.assertRaises(
+            RuntimeError,
+            app.get,
+            '/testing/'
+        )
+
+    def test_conflicting_custom_routes_in_subclass(self):
+
+        class BaseController(object):
+
+            @expose(route='testing')
+            def foo(self):
+                return request.path
+
+        class ChildController(BaseController):
+            pass
+
+        class RootController(BaseController):
+            child = ChildController()
+
+        app = TestApp(Pecan(RootController()))
+
+        r = app.get('/testing/')
+        assert r.body == b_('/testing/')
+
+        r = app.get('/child/testing/')
+        assert r.body == b_('/child/testing/')
+
+    def test_custom_route_prohibited_on_lookup(self):
+        try:
+            class RootController(object):
+
+                @expose(route='some-path')
+                def _lookup(self):
+                    return 'Hello, World!'
+        except ValueError:
+            pass
+        else:
+            raise AssertionError(
+                '_lookup cannot be used with a custom path segment'
+            )
+
+    def test_custom_route_prohibited_on_default(self):
+        try:
+            class RootController(object):
+
+                @expose(route='some-path')
+                def _default(self):
+                    return 'Hello, World!'
+        except ValueError:
+            pass
+        else:
+            raise AssertionError(
+                '_default cannot be used with a custom path segment'
+            )
+
+    def test_custom_route_prohibited_on_route(self):
+        try:
+            class RootController(object):
+
+                @expose(route='some-path')
+                def _route(self):
+                    return 'Hello, World!'
+        except ValueError:
+            pass
+        else:
+            raise AssertionError(
+                '_route cannot be used with a custom path segment'
+            )
+
+    def test_custom_route_with_generic_controllers(self):
+
+        class RootController(object):
+
+            @expose(route='some-path', generic=True)
+            def foo(self):
+                return 'Hello, World!'
+
+            @foo.when(method='POST')
+            def handle_post(self):
+                return 'POST!'
+
+        app = TestApp(Pecan(RootController()))
+
+        r = app.get('/some-path/')
+        assert r.status_int == 200
+        assert r.body == b_('Hello, World!')
+
+        r = app.get('/foo/', expect_errors=True)
+        assert r.status_int == 404
+
+        r = app.post('/some-path/')
+        assert r.status_int == 200
+        assert r.body == b_('POST!')
+
+        r = app.post('/foo/', expect_errors=True)
+        assert r.status_int == 404
+
+    def test_custom_route_prohibited_on_generic_controllers(self):
+        try:
+            class RootController(object):
+
+                @expose(generic=True)
+                def foo(self):
+                    return 'Hello, World!'
+
+                @foo.when(method='POST', route='some-path')
+                def handle_post(self):
+                    return 'POST!'
+        except ValueError:
+            pass
+        else:
+            raise AssertionError(
+                'generic controllers cannot be used with a custom path segment'
+            )
+
+    def test_invalid_route_arguments(self):
+        class C(object):
+
+            def secret(self):
+                return {}
+
+        self.assertRaises(TypeError, route)
+        self.assertRaises(TypeError, route, 'some-path', lambda x: x)
+        self.assertRaises(TypeError, route, 'some-path', C.secret)
+        self.assertRaises(TypeError, route, C, {}, C())
+
+        for path in (
+            'VARIED-case-PATH',
+            'this,custom,path',
+            '123-path',
+            'path(with-parens)',
+            'path;with;semicolons',
+            'path:with:colons',
+        ):
+            handler = C()
+            route(C, path, handler)
+            assert getattr(C, path, handler)
+
+        self.assertRaises(ValueError, route, C, '/path/', C())
+        self.assertRaises(ValueError, route, C, '.', C())
+        self.assertRaises(ValueError, route, C, '..', C())
+        self.assertRaises(ValueError, route, C, 'path?', C())
+        self.assertRaises(ValueError, route, C, 'percent%20encoded', C())
